function B = tpbspl(x,k,t1,t2,der)
% B = tpbspl(x,k,t1,t2,der)
% Bivariate tensor-product B-spline basis
% Input:
%   x   (n x 2)     Points in R^2 where the basis will be evaluated
%   k   (scalar)    Spline order (use 4 for cubic)
%   t1  (p1 x 1)    Knot sequence on the first variable
%                    (must include interval endpoints)
%   t2  (p2 x 1)    Knot sequence on the second variable
%                    (must include interval endpoints)
%   der (scalar)    Derivative order of the basis functions
%                    (optional argument; default is 0)
% Output: Tensor-product basis functions or derivatives evaluated at x,
%           depending on input DER
%   If der = 0: B  (n x p)      Basis function values
%   If der = 1: B  (n x p x 2)  B(:,:,j) is first deriv wrt x_j
%   If der = 2: B  (n x p x 3)  Second derivatives, B(:,:,1) wrt (x_1,x_1),
%                       B(:,:,2) wrt (x_1,x_2), B(:,:,3) wrt (x_2,x_2)
%
% External calls: BSPL
% Version: November 2019

if nargin<5
    der = 0;
end
p1 = length(t1)+k-2;
p2 = length(t2)+k-2;
i1 = kron(1:p1,ones(1,p2));
i2 = kron(ones(1,p1),1:p2);

switch der
    case 0
        B1 = bspl(x(:,1),k,t1,0);
        B2 = bspl(x(:,2),k,t2,0);
        B = B1(:,i1).*B2(:,i2);
    case 1
        B10 = bspl(x(:,1),k,t1,0);
        B20 = bspl(x(:,2),k,t2,0);
        B11 = bspl(x(:,1),k,t1,1);
        B21 = bspl(x(:,2),k,t2,1);
        B(:,:,1) = B11(:,i1).*B20(:,i2);
        B(:,:,2) = B10(:,i1).*B21(:,i2);
    case 2
        B10 = bspl(x(:,1),k,t1,0);
        B20 = bspl(x(:,2),k,t2,0);
        B11 = bspl(x(:,1),k,t1,1);
        B21 = bspl(x(:,2),k,t2,1);
        B12 = bspl(x(:,1),k,t1,2);
        B22 = bspl(x(:,2),k,t2,2);
        B(:,:,1) = B12(:,i1).*B20(:,i2);
        B(:,:,2) = B11(:,i1).*B21(:,i2);
        B(:,:,3) = B10(:,i1).*B22(:,i2);
    otherwise
        B = [];
        disp('Wrong input DER -- must be 0, 1 or 2')
end

