function [A,D,Sigma] = MeanCov_t(x,basis)
% [A,D,Sigma] = MeanCov_t(x,basis)
% Mean and covariance estimators of temporal intensity functions 
%   for multivariate DSPPs
% 
% Input:
%   x   (n x d, cell)   Point process observations
%                       (n are the replications, d are the dimensions)
%   basis   (struct)    B-spline basis parameters. Struct fields:
%       rng (1 x 2)     Time range
%       or  (scalar)    Spline order (4 is cubic splines)
%       nk  (scalar)    Number of knots (knots will be equally spaced)
% Output:
%   A       (p x d)     B-spline coefficients of mean functions
%   D   (p x p x d x d) B-spline coefficients of covariance functions
%   Sigma   (d x d)     Spatial covariances
%
% Note: to obtain Mu and Cov's from coefficients, if b(t) is the B-spline
%   basis vector evaluated at t, then Mu_j(t) = b(t)'*A(:,j) and
%   Cov_(j,k)(t1,t2) = b(t1)'*D(:,:,j,k)*b(t2)
%
% External calls: BSPL
% Version: March 2021

% Initialization
[n,d] = size(x);
p = basis.or+basis.nk;
ngrid = 5000;
knt = linspace(basis.rng(1),basis.rng(2),basis.nk+2);
t = linspace(basis.rng(1),basis.rng(2),ngrid);
B0 = bspl(t,basis.or,knt,0);
IBB = (B0'*B0)*(t(2)-t(1));

% Basis function evaluation
B = cell(n,d);
for i = 1:n
    for j = 1:d
        B{i,j} = bspl(x{i,j},basis.or,knt,0);
    end
end

% Mean estimator coefficients
A = zeros(p,d);
for j = 1:d
    sum_B = zeros(p,1);
    for i = 1:n
        sum_B = sum_B + sum(B{i,j},1)';
    end
    A(:,j) = IBB\(sum_B/n);
end

% Second-moment estimator coefficients
C = zeros(p,p,d,d);
for j = 1:d
    for k = j   % case j=k
        sumBB = zeros(p,p);
        for i = 1:n
            mij = length(x{i,j});
            for l = 1:mij
                idif = x{i,j}~=x{i,j}(l);
                sumBB = sumBB + B{i,j}(l,:)'*sum(B{i,j}(idif,:),1);
            end
        end
        C(:,:,j,j) = (IBB\(sumBB/n))/IBB;
    end
    for k = (j+1):d     % case j~=k
        sumBB = zeros(p,p);
        for i = 1:n
            sumBB = sumBB + sum(B{i,j},1)'*sum(B{i,k},1);
        end
        C(:,:,j,k) = (IBB\(sumBB/n))/IBB;
        C(:,:,k,j) = C(:,:,j,k);
    end
end

% Covariance estimator coefficients
D = zeros(p,p,d,d);
for j = 1:d
    D(:,:,j,j) = C(:,:,j,j) - A(:,j)*A(:,j)';
    for k = (j+1):d
        D(:,:,j,k) = C(:,:,j,k) - A(:,j)*A(:,k)';
        D(:,:,k,j) = D(:,:,j,k);
    end
end

% Spatial covariance estimators
Sigma = zeros(d,d);
for j = 1:d
    Sigma(j,j) = trace(D(:,:,j,j)*IBB);
    for k = (j+1):d
        Sigma(j,k) = trace(D(:,:,j,k)*IBB);
        Sigma(k,j) = Sigma(j,k);
    end
end
