function cf = kriging(s0,s,A,Sigma,B,C,basis_t,basis_s)
% cf = kriging(s0,s,A,Sigma,B,C,basis_t,basis_s)
% Kriging predictor for spatio-temporal fields (trimmed Sigma)
% Input:
%   s0      (1 x 2)     Spatial location for prediction
%   s       (d x 2)     Observed spatial locations
%   A       (p x d)     Basis coefficients of means (output of MeanCov_t)
%   Sigma   (d x d)     Spatial variance-covariances (output of MeanCov_t)
%   B       (p x q)     Basis coefficients of smoothed mean (output of spsmMean)
%   C       (q x q)     Basis coefficients of smoothed covariances (output of spsmCov)
%   basis_t (struct)    Temporal spline basis parameters. Struct fields:
%       rng (1 x 2)         Time range
%       or  (scalar)        Spline order (4 is cubic splines)
%       nk  (scalar)        Number of knots (knots will be equally spaced)
%   basis_s (struct)    Spatial tensor-B-spline basis parameters. Fields:
%       rg1 (1 x 2)         Range of first spatial coordinate (longitude)
%       rg2 (1 x 2)         Range of second spatial coordinate (latitude)
%       or  (scalar)        Spline order (4 is cubic splines)
%       nk  (scalar)        Number of (equispaced) knots per coordinate
%
% Output:
%   cf      (d x 1)     Kriging coefficients
%
% External calls: BSPL, TPBSPL
% Version: June 2021

ngrid = 500;
knt = linspace(basis_t.rng(1),basis_t.rng(2),basis_t.nk+2);
t = linspace(basis_t.rng(1),basis_t.rng(2),ngrid);
B0 = bspl(t,basis_t.or,knt,0);
IBB = (B0'*B0)*(t(2)-t(1));
t1 = linspace(basis_s.rg1(1),basis_s.rg1(2),basis_s.nk+2);
t2 = linspace(basis_s.rg2(1),basis_s.rg2(2),basis_s.nk+2);
G = tpbspl(s,basis_s.or,t1,t2,0);
g0 = tpbspl(s0,basis_s.or,t1,t2,0);

sigma_0 = G*C*g0';
M = A'*IBB*A;
m_0 = A'*IBB*B*g0';

[U,S] = svd(M);
r = find(cumsum(diag(S))/sum(diag(S))>.90,1);
MM = S(1:r,1:r)*U(:,1:r)';
mm_0 = U(:,1:r)'*m_0;

[U,S] = svd(Sigma);
tS = find(cumsum(diag(S))/sum(diag(S))>.90,1);
Sigma2 = U(:,1:tS)*S(1:tS,1:tS)*U(:,1:tS)';

H = [U(:,1:tS)'*Sigma2*U(:,1:tS), U(:,1:tS)'*MM'; MM*U(:,1:tS), zeros(r)];
v = [U(:,1:tS)'*sigma_0; mm_0];
x = H\v;

cf = U(:,1:tS)*x(1:tS);
