function [C,mse,df,gcv] = spsmCov(s,Sigma,basis,lmb)
% function [C,mse,df,gcv] = spsmCov(s,Sigma,basis,lmb)
% Spatial smoothing of covariance field for multivariate DSPPs
% Input:
%   s       (d x 2)     Spatial locations
%   Sigma   (d x d)     Spatial covariances (output of MeanCov_t)
%   basis   (struct)    Spatial tensor-B-spline basis parameters. Fields:
%       rg1 (1 x 2)     Range of first spatial coordinate (longitude)
%       rg2 (1 x 2)     Range of second spatial coordinate (latitude)
%       or  (scalar)    Spline order (4 is cubic splines)
%       nk  (scalar)    Number of (equispaced) knots per coordinate
%   lmb     (scalar)    Smoothing parameter
%
% Output:
%   C       (q x q)     Basis coefficients for Sigma
%                        (Sigma_jk = g(s_j)'*C*g(s_k), where g(s) is the 
%                         spatial basis)
%   mse     (scalar)    Mean squared error
%   df      (scalar)    Smoothing degrees of freedom
%   gcv     (scalar)    Generalized cross-validation MSE
%
% External calls: TPBSPL
% Version: June 2021

% Basis evaluation
t1 = linspace(basis.rg1(1),basis.rg1(2),basis.nk+2);
t2 = linspace(basis.rg2(1),basis.rg2(2),basis.nk+2);
G = tpbspl(s,basis.or,t1,t2,0);

% Roughness penalty matrix
ng = 500;
J = PenMat(ng,basis);

% Estimator
d = size(G,1);
[U,S,V] = svd(G,'econ');
S1 = diag(1./diag(S));
[W,F] = svd(S1*V'*J*V*S1);
D1 = reshape(1./(1+lmb*kron(diag(F),diag(F))),d,d);
A = W'*U'*(Sigma-diag(diag(Sigma)))*U*W;
v = (D1.*A) + D1.*proj(D1.*A,U*W);
C = V*S1*W*reshape(v,d,d)*W'*S1*V';
df = dfC(D1,U*W);
mse = (sum(sum((Sigma-G*C*G').^2))-sum((diag(Sigma)-diag(G*C*G')).^2))/(d*(d-1));
gcv = mse/(1-df/(d*(d-1)))^2;
end

%%% -------------------------

function J = PenMat(ng,basis)
% Roughness penalty matrix
t1 = linspace(basis.rg1(1),basis.rg1(2),basis.nk+2);
t2 = linspace(basis.rg2(1),basis.rg2(2),basis.nk+2);
sg1 = linspace(basis.rg1(1),basis.rg1(2),ng)';
sg2 = linspace(basis.rg2(1),basis.rg2(2),ng)';
B10 = bspl(sg1,basis.or,t1,0);
B20 = bspl(sg2,basis.or,t2,0);
B11 = bspl(sg1,basis.or,t1,1);
B21 = bspl(sg2,basis.or,t2,1);
B12 = bspl(sg1,basis.or,t1,2);
B22 = bspl(sg2,basis.or,t2,2);
J10 = (B10'*B10)*(sg1(2)-sg1(1));
J20 = (B20'*B20)*(sg2(2)-sg2(1));
J11 = (B11'*B11)*(sg1(2)-sg1(1));
J21 = (B21'*B21)*(sg2(2)-sg2(1));
J12 = (B12'*B12)*(sg1(2)-sg1(1));
J22 = (B22'*B22)*(sg2(2)-sg2(1));
J = kron(J12,J20) + 2*kron(J11,J21) + kron(J10,J22);
end

function M = proj(A,U)
d = size(A,1);
M = zeros(d,d);
for j = 1:d
    M = M + U(j,:)'*U(j,:)*A*U(j,:)'*U(j,:);
end
end

function df = dfC(D1,V)
d = size(D1,1);
diagP = zeros(d^2,1);
for j = 1:d
    diagP = diagP + kron((V(j,:).^2)',(V(j,:).^2)');
end
df = sum(D1(:)) + sum(D1(:).*diagP.*D1(:));
end


