function [C,lmb0,mse0,df0,gcv0,mse,df,gcv] = spsmCov_gcv(s,Sigma,basis,lmb)
% [C,lmb0,mse0,df0,gcv0,mse,df,gcv] = spsmCov_gcv(s,Sigma,basis,lmb)
% Spatial smoothing of covariance field for multivariate DSPPs
%  Chooses optimal (minimum gcv) estimator on a grid of possible lmb's
% Input:
%   s       (d x 2)     Spatial locations
%   Sigma   (d x d)     Spatial covariances (output of MeanCov_t)
%   basis   (struct)    Spatial tensor-B-spline basis parameters. Fields:
%       rg1 (1 x 2)     Range of first spatial coordinate (longitude)
%       rg2 (1 x 2)     Range of second spatial coordinate (latitude)
%       or  (scalar)    Spline order (4 is cubic splines)
%       nk  (scalar)    Number of (equispaced) knots per coordinate
%   lmb     (1 x ng)    Smoothing parameters
%
% Output:
%   C       (q x q)     Basis coefficients for Sigma
%                        (Sigma_jk = g(s_j)'*C*g(s_k), where g(s) is the 
%                         spatial basis)
%   lmb0    (scalar)    Optimal lmb
%   mse0    (scalar)    Optimal mean squared error
%   df0     (scalar)    Optimal smoothing degrees of freedom
%   gcv0    (scalar)    Optimal generalized cross-validation MSE
%   mse     (1 x ng)    Mean squared error for each lmb
%   df      (1 x ng)    Smoothing degrees of freedom for each lmb
%   gcv     (1 x ng)    Generalized cross-validation MSE for each lmb
%
% External calls: spsmCov
% Version: March 2021

% Find best estimator
for k = 1:length(lmb)
    try
        [C,mse(k),df(k),gcv(k)] = spsmCov(s,Sigma,basis,lmb(k));
    catch ME
        C = [];
        mse(k) = NaN;
        df(k) = NaN;
        gcv(k) = NaN;
    end
end
% Recompute best estimator
if any(isfinite(gcv))
    [mgcv,kmin] = min(gcv);
    lmb0 = lmb(kmin);
    [C,mse0,df0,gcv0] = spsmCov(s,Sigma,basis,lmb0);
else
    disp('No estimators were finite on lmb grid')
    disp('Therefore returning empty outputs')
    C = [];
    mse = [];
    df = [];
    gcv = [];
    lmb0 = [];
    mse0 = [];
    df0 = [];
    gcv0 = [];
end
