function [B,mse,df,gcv] = spsmMean(s,A,basis,lmb)
% function [B,mse,df,gcv] = spsmMean(s,A,basis,lmb)
% Spatial smoothing of mean coefficients for multivariate DSPPs
% Input:
%   s       (d x 2)     Spatial locations
%   A       (p x d)     B-spline coeffs of means (output of MeanCov_t)
%   basis   (struct)    Spatial tensor-B-spline basis parameters. Fields:
%       rg1 (1 x 2)     Range of first spatial coordinate (longitude)
%       rg2 (1 x 2)     Range of second spatial coordinate (latitude)
%       or  (scalar)    Spline order (4 is cubic splines)
%       nk  (scalar)    Number of (equispaced) knots per coordinate
%   lmb     (scalar)    Smoothing parameter
%
% Output:
%   B       (p x q)     Basis coefficients for mu(t,s)
%                        (mu(t,s) = b(t)'*B*g(s), where b(t) is the 
%                         temporal basis and g(s) the spatial basis)
%   mse     (scalar)    Mean squared error
%   df      (scalar)    Smoothing degrees of freedom
%   gcv     (scalar)    Generalized cross-validation MSE
%
% External calls: TPBSPL
% Version: March 2021

% Basis evaluation
t1 = linspace(basis.rg1(1),basis.rg1(2),basis.nk+2);
t2 = linspace(basis.rg2(1),basis.rg2(2),basis.nk+2);
G = tpbspl(s,basis.or,t1,t2,0);

% Roughness penalty matrix
ng = 500;
J = PenMat(ng,basis);

% Estimator
B = (A*G)/(G'*G + lmb*J);
df = trace((G'*G + lmb*J)\(G'*G));
d = size(s,1);
mse = norm(A-B*G','fro')^2/d;
gcv = norm(A-B*G','fro')^2/(d*(1-df/d)^2);
end

%%% -------------------------

function J = PenMat(ng,basis)
% Roughness penalty matrix
t1 = linspace(basis.rg1(1),basis.rg1(2),basis.nk+2);
t2 = linspace(basis.rg2(1),basis.rg2(2),basis.nk+2);
sg1 = linspace(basis.rg1(1),basis.rg1(2),ng)';
sg2 = linspace(basis.rg2(1),basis.rg2(2),ng)';
B10 = bspl(sg1,basis.or,t1,0);
B20 = bspl(sg2,basis.or,t2,0);
B11 = bspl(sg1,basis.or,t1,1);
B21 = bspl(sg2,basis.or,t2,1);
B12 = bspl(sg1,basis.or,t1,2);
B22 = bspl(sg2,basis.or,t2,2);
J10 = (B10'*B10)*(sg1(2)-sg1(1));
J20 = (B20'*B20)*(sg2(2)-sg2(1));
J11 = (B11'*B11)*(sg1(2)-sg1(1));
J21 = (B21'*B21)*(sg2(2)-sg2(1));
J12 = (B12'*B12)*(sg1(2)-sg1(1));
J22 = (B22'*B22)*(sg2(2)-sg2(1));
J = kron(J12,J20) + 2*kron(J11,J21) + kron(J10,J22);
end