function [B,lmb0,mse0,df0,gcv0,mse,df,gcv] = spsmMean_gcv(s,A,basis,lmb)
% [B,lmb0,mse0,df0,gcv0,mse,df,gcv] = spsmMean_gcv(s,A,basis,lmb)
% Spatial smoothing of mean coefficients for multivariate DSPPs
%  Chooses optimal (minimum gcv) estimator on a grid of possible lmb's
% Input:
%   s       (d x 2)     Spatial locations
%   A       (p x d)     B-spline coeffs of means (output of MeanCov_t)
%   basis   (struct)    Spatial tensor-B-spline basis parameters. Fields:
%       rg1 (1 x 2)     Range of first spatial coordinate (longitude)
%       rg2 (1 x 2)     Range of second spatial coordinate (latitude)
%       or  (scalar)    Spline order (4 is cubic splines)
%       nk  (scalar)    Number of (equispaced) knots per coordinate
%   lmb     (1 x ng)    Smoothing parameters
%
% Output:
%   B       (p x q)     Basis coefficients for mu(t,s)
%                        (mu(t,s) = b(t)'*B*g(s), where b(t) is the 
%                         temporal basis and g(s) the spatial basis)
%   lmb0    (scalar)    Optimal lmb
%   mse0    (scalar)    Optimal mean squared error
%   df0     (scalar)    Optimal smoothing degrees of freedom
%   gcv0    (scalar)    Optimal generalized cross-validation MSE
%   mse     (1 x ng)    Mean squared error for each lmb
%   df      (1 x ng)    Smoothing degrees of freedom for each lmb
%   gcv     (1 x ng)    Generalized cross-validation MSE for each lmb
%
% External calls: spsmMean
% Version: March 2021

% Find best estimator
for k = 1:length(lmb)
    try
        [B,mse(k),df(k),gcv(k)] = spsmMean(s,A,basis,lmb(k));
    catch ME
        B = [];
        mse(k) = NaN;
        df(k) = NaN;
        gcv(k) = NaN;
    end
end
% Recompute best estimator
if any(isfinite(gcv))
    [mgcv,kmin] = min(gcv);
    lmb0 = lmb(kmin);
    [B,mse0,df0,gcv0] = spsmMean(s,A,basis,lmb0);
else
    disp('No estimators were finite on lmb grid')
    disp('Therefore returning empty outputs')
    B = [];
    mse = [];
    df = [];
    gcv = [];
    lmb0 = [];
    mse0 = [];
    df0 = [];
    gcv0 = [];
end
